{ SVGA256 V2.1 English                                                       }
{ Unit for 256-colors graphics                                               }
{ Needs following files:                                                     }
{  SVGA256.BGI   -  Borland Graphics Interface for VESA cards                }
{ (c) Matus Horvath, 1997, sjs2@isternet.sk                                  }
unit SVGA256;
{-[ INTERFACE ]----------------------------------------------------/////-----}
 interface
{-----[ Type DacPalette256 for palette editing ]-------------------/////-----}
  type DacPalette256 = array[0..255] of array[0..2] of Byte;
{-----[ Type VesaInfo - info about graphics card ]-----------------/////-----}
       VESAInfo=record
                 VESAVer_Hi:byte;
                 VESAVer_Lo:byte;
                 Manufacturer:string;
                 Modes:byte;
                end;

{-----[ Constants for graphics initialization ]--------------------/////-----}
  const VGA320x200x256    = 0;  {  320x200x256 }
        V32256            = 0;
        SVGA640x400x256   = 1;  {  640x400x256 }
        S63256            = 1;
        SVGA640x480x256	  = 2;  {  640x480x256 }
        S64256            = 2;
        SVGA800x600x256   = 3;  {  800x600x256 }
        S86256            = 3;
        SVGA1024x768x256  = 4;  { 1024x768x256 }
        S17256            = 4;
{-----[ Constants for paths to BGI and TP directories ]------------/////-----}
        Bgi    = 'C:\TP\BGI\';  { BGI Directory  }
        TP     = 'C:\TP\';      { TP Directory  }
{-----[ Veriables ]------------------------------------------------/////-----}
  var SVGA:integer;             { Drivera number for InitGraph }
      MaxSVGAMode:integer;      { Maxium mode# detected by InitGraph }
      SVGAInfo:VESAInfo;

{-----[ Initialization ]-------------------------------------------/////-----}
{-------[ Procedure INITIALIZE compatible with SVGA256 1.x ]-------/////-----}
  procedure Initialize(gm:integer;pd:string);
{-------[ Procedures for faster initialization ]-------------------/////-----}
  procedure I32256;    {  320x200x256 }
  procedure I63256;    {  640x400x256 }
  procedure I64256;    {  640x480x256 }
  procedure I86256;    {  800x600x256 }
  procedure I17256;    { 1024x768x256 }
{-------[ Palette editing ]----------------------------------------/////-----}
  procedure SetVGAPalette(PalBuf : DacPalette256);
{-[ IMPLEMENTATION ]-----------------------------------------------/////-----}
 implementation
  uses Crt, Dos, Graph, Strings;
{-----[ Private types for card info ]------------------------------/////-----}
  type VModes=array[1..238]of word;
       InfoBuffer=record
                   VESASig:array[1..4]of char;
                   VESAVer_Hi:byte;
                   VESAVer_Lo:byte;
                   Manufacturer:pchar;
                   Flag:longint;
                   VideoModes:^vmodes;
                   FreeSpace:array[1..119]of word;
                  end;
     P_InfoBuffer=^InfoBuffer;
{-----[ Private types for mode info ]------------------------------/////-----}
     ModeInfoBuffer=record
                     ModeFlag:word;
                     NotUsed1:array[1..16]of byte;
                     XResolution:word;
                     YResolution:word;
                     NotUsed2:array[1..3]of byte;
                     BitesPerDot:byte;
                     NotUsed3:array[1..230]of byte;
                    end;
     P_ModeInfoBuffer=^ModeInfoBuffer;

{-----[ Card detection procedures ]--------------------------------/////-----}
{-------[ Gets card info ]-----------------------------------------/////-----}
 function GetVESAInfo(var P:InfoBuffer):boolean;
  {-----[ Parametres ]------------------------/////-----}
  { P = record, in which info will be written           }
  {-----[ Output ]----------------------------/////-----}
  { Did the function execute properly? TRUE=yes.        }
  var r:registers;
  begin
   r.ah:=$4f;
   r.al:=$00;
   r.es:=seg(p);
   r.di:=ofs(p);
   intr($10,r);
   getvesainfo:=(r.al=$4f)and(r.ah=$00);
  end;

{-------[ Gets mode info ]-----------------------------------------/////-----}
 function GetModeInfo(Mode:word;var P:ModeInfoBuffer):boolean;
  {-----[ Parametres ]------------------------/////-----}
  { Mode = mode#                                        }
  { P    = record, where mode info will be written      }
  {-----[ Output ]----------------------------/////-----}
  { Did function execute properly? TRUE=ano.            }
  var r:registers;
  begin
   r.ah:=$4f;
   r.al:=$01;
   r.cx:=mode;
   r.es:=seg(p);
   r.di:=ofs(p);
   intr($10,r);
   getmodeinfo:=(r.ah=$00)and(r.al=$4f);
  end;

{-------[ Tests, whether is a mode supported ]---------------------/////-----}
 function Je_To_Nas_Mod(X,Y,B:word;PMI:ModeInfoBuffer):boolean;
  {-----[ Parametres ]------------------------/////-----}
  { X,Y = tested resolution                             }
  { B   = tested color depth (bits per pixel)           }
  { PMI = tested graph mode                             }
  {-----[ Vystup ]----------------------------/////-----}
  { Does tested mode have resolution X,Y and color      }
  { color depth B? TRUE=yes                             }
  begin
   {   X-res                 Y-res                      }
   if (pmi.xresolution=x)and(pmi.yresolution=y)and
   {   colors                 mode                      }
      (pmi.bitesperdot=b)and((pmi.modeflag and 16)=16)and
   {   color mode                  monitor ok           }
      ((pmi.modeflag and 8)=8)and((pmi.modeflag and 1)=1) then
   {  return true, if it is our mode}
      je_to_nas_mod:=true else je_to_nas_mod:=false;
  end;

 procedure Nuluj(var VI:VESAInfo);
  begin
   VI.VESAVer_HI:=0;
   VI.VESAVer_LO:=0;
   VI.Manufacturer:='Card was not detected yet/correctly.';
   VI.Modes:=0;
  end;

{-------[ Gets info about graphics card ]--------------------------/////-----}
 function DetectVESA(var VI:VESAInfo):boolean;
  {-----[ Parametres ]-------------------------/////--------}
  { VI  = record, where card and modes info will be written }
  {-----[ Output ]-----------------------------/////--------}
  { Did we find a VGA/MCGA/VESA card?                       }
  { TRUE=yes.                                               }
  var a:word;
      gd,gm:integer;
      pmi:modeinfobuffer;
      pi:infobuffer;
  begin
   nuluj(vi);
   {is it at least VGA or MCGA}
   detectgraph(gd,gm);
   if (gd=VGA)or(gd=MCGA) then begin
    vi.modes:=vi.modes or $01;
    {is it VESA?}
    if getvesainfo(pi)and(pi.vesasig[1]+pi.vesasig[2]+pi.vesasig[3]+
                                        pi.vesasig[4]='VESA')then begin
     {it is VESA, what version?}
     vi.vesaver_lo:=pi.vesaver_lo;
     vi.vesaver_hi:=pi.vesaver_hi;
     vi.manufacturer:=strpas(pi.manufacturer);

     a:=1;
     {for every mode, or every lower then 238}
     while ((a<=238)and(pi.videomodes^[a]<>$ffff)) do begin
      {if you have mode info,}
      if getmodeinfo(pi.videomodes^[a],pmi) then begin
       {and resolution, color, etc info,}
       if (pmi.modeflag and 2)=2 then begin
        {have resolution info, use it}
        {640x400x256}
        if je_to_nas_mod(640,400,8,pmi) then vi.modes:=vi.modes or $02;
        {640x480x256}
        if je_to_nas_mod(640,480,8,pmi) then vi.modes:=vi.modes or $04;
        {800x600x256}
        if je_to_nas_mod(800,600,8,pmi) then vi.modes:=vi.modes or $08;
        {1024x768x256}
        if je_to_nas_mod(1024,768,8,pmi) then vi.modes:=vi.modes or $10;
       end else begin
       {don't have resolution info, try to use mede numbers}
        if pi.videomodes^[a]=$100 then vi.modes:=vi.modes or $02;
        if pi.videomodes^[a]=$101 then vi.modes:=vi.modes or $04;
        if pi.videomodes^[a]=$103 then vi.modes:=vi.modes or $08;
        if pi.videomodes^[a]=$105 then vi.modes:=vi.modes or $10;
       end;
      end;
      inc(a);
     end;
    end else begin {it isn't VESA, but is VGA/MCGA}
     {Set card info}
     vi.vesaver_lo:=0;
     vi.vesaver_hi:=0;
     vi.manufacturer:='VGA or MCGA standart';
    end;
    detectvesa:=true
   end else detectvesa:=false; {It isn't VGA}
  end;

{-----[ Procedure, which detects VGA/MCGA/VESA + optimal mode# ]---/////-----}
  {$F+}
  function DetectSVGA256: integer;
  {-----[ Parametres ]-------------------------/////-----}
  { No                                                   }
  {-----[ Output ]-----------------------------/////-----}
  { Optimal mode# or grError if no VGA/MCGA/VESA is      }
  { present.                                             }
   begin
    if DetectVesa(svgainfo) then begin
     if ((svgainfo.modes and 01)=01) then DetectSVGA256:=0;
     if ((svgainfo.modes and 02)=02)and(MaxSVGAMode>=SVGA640x400x256)
        then DetectSVGA256:=1;
     if ((svgainfo.modes and 04)=04)and(MaxSVGAMode>=SVGA640x480x256)
        then DetectSVGA256:=2;
     if ((svgainfo.modes and 08)=08)and(MaxSVGAMode>=SVGA800x600x256)
        then DetectSVGA256:=3;
     if ((svgainfo.modes and 16)=16)and(MaxSVGAMode>=SVGA1024x768x256)
        then DetectSVGA256:=4;
    end else DetectSVGA256:=grError;
   end; { DetectSVGA256 }
   {$F-}

{-----[ Gr. mode init compatible with SVGA256 1.x ]----------------/////-----}
  procedure Initialize(gm:integer;pd:string);
  {-----[ Parametres ]-------------------------/////-----}
  { GM = Mode#                                           }
  { PD = Path to SVGA256.BGI                             }
  { This one was copied from one file at school, but the }
  { original had several mistakes.                       }
   var PathToDriver:string;
       ErrorCode:integer;
   begin
    DirectVideo:=False;
    PathToDriver:=PD;
    repeat
     InitGraph(SVGA,GM,PathToDriver);
     ErrorCode:=GraphResult;
     if ErrorCode<>grOK then begin
      Writeln('Graphics error: ', GraphErrorMsg(ErrorCode));
      if ErrorCode=grFileNotFound then begin
       Writeln('Enter full path to BGI driver or type <Ctrl-Break> to quit:');
       Readln(PathToDriver);
       Writeln;
      end else Halt(1);
     end;
    until ErrorCode=grOK;
    setcolor(white);
   end; { Initialize }

{-----[ Palette edit ]---------------------------------------------/////-----}
  procedure SetVGAPalette(PalBuf:DacPalette256);
  {-----[ Parametres ]------------------------/////-----}
  { PalBuf = New palette                                }
   var Reg:Registers;
   begin
    reg.ax := $1012;
    reg.bx := 0;
    reg.cx := 256;
    reg.es := Seg(PalBuf);
    reg.dx := Ofs(PalBuf);
    intr($10,reg);
   end;

{-----[ Fast mode init ]-------------------------------------------/////-----}
  procedure I32256;
   begin
    initialize(0,bgi);
   end;

  procedure I63256;
   begin
    initialize(1,bgi);
   end;

  procedure I64256;
   begin
    initialize(2,bgi);
   end;

  procedure I86256;
   begin
    initialize(3,bgi);
   end;

  procedure I17256;
   begin
    initialize(4,bgi);
   end;

 begin
  MaxSVGAMode:=SVGA800x600x256;
  SVGA:=DetectSVGA256;
  SVGA:=InstallUserDriver('SVGA256', @DetectSVGA256);
  if SVGA=grError then SVGA:=Detect;
 end.
